@@ -22,8 +22,14 @@
  *									* 
  ************************************************************************/
 
-/* $Id: rpncalc.c,v 1.0 1995/12/31 18:14:13 david Rel $
+/* $Id: rpncalc.c,v 1.2 1996/09/13 20:21:29 david Rel $
  * $Log: rpncalc.c,v $
+ * Revision 1.2  1996/09/13 20:21:29  david
+ * lclint additions
+ *
+ * Revision 1.1  1996/07/13 20:53:44  david
+ * Added operator completion, renaming due to linting
+ *
  * Revision 1.0  1995/12/31 18:14:13  david
  * Initial revision
  * */ 
@@ -50,56 +56,57 @@ extern int pushtostack;
 
 char *progname;
 
-static struct option const long_options[] =
+struct option const long_options[] =
 {
-  {"help", no_argument, 0, 'h'},
-  {"help", no_argument, 0, '?'},
+  {"help",    no_argument, 0, 'h'},
+  {"help",    no_argument, 0, '?'},
   {"version", no_argument, 0, 'v'},
-  {0, 0, 0, 0}
+  {(char *)0, 0,           0, (char)0}
 };
 
 double help(void)
 {
-  printf("The following operations will be emulated:\n");
-  printf("push  pop  pick  swap  over  roll  dup  dupn  drop  dropn  depth"\
-	 "\n\n");
-  printf("chs   +    -     *     /     ^     inv  sqrt  sqr\n");
-  printf("sin   cos  tan   asin  acos  atan  atan2\n");
-  printf("sinh  cosh tanh  asinh acosh atanh\n");
-  printf("ln    log  ld    exp   alog  shl\n");
-  printf("j0    j1   jn    y0    y1    yn\n");
-  printf("erf   erfc lgamma\n");
-  printf("abs   ceil fact  mod  div    gcd   sum  prod\n\n");
-  printf("hex   dec  oct   and  &      or    |    not   !     xor\n");
-  printf("prec\n");
-  printf("The following constants will be recognised: pi, e.\n");
-  printf("Delimiters are ',', ';', space, tab and newline.\n");
-  printf("Case is significant. Use prefix 0x for hexadecimal "\
-	 "and 0 for octal constants.\n");
+#define HELP "\
+The following operations will be emulated:\n\
+  push  pop  pick  swap  over  roll  dup  dupn  drop  dropn  depth\n\
+\n\
+  chs   +    -     *     /     ^     inv  sqrt  sqr\n\
+  sin   cos  tan   asin  acos  atan  atan2\n\
+  sinh  cosh tanh  asinh acosh atanh\n\
+  ln    log  ld    exp   alog  shl\n\
+  j0    j1   jn    y0    y1    yn\n\
+  erf   erfc lgamma\n\
+  abs   ceil fact  mod  div    gcd   sum  prod\n\
+\n\
+  hex   dec  oct   and  &      or    |    not   !     xor\n\
+  prec\n\
+\n\
+  The following constants will be recognised: pi, e.\n\
+  Delimiters are ',', ';', space, tab and newline.\n"
+
+  printf(HELP);
 
   return 0.0; /* dummy value */
 }
 
 double warranty(void)
 {
-  printf("This program is free software; you can redistribute it and/or "\
-	 "modify\n");
-  printf("it under the terms of the GNU General Public License as published "\
-	 "by\n");
-  printf("the Free Software Foundation; either version 2 of the License, "\
-	 "or\n");
-  printf("(at your option) any later version.\n");
-  printf("\n");
-  printf("This program is distributed in the hope that it will be useful,\n");
-  printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
-  printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
-  printf("GNU General Public License for more details.\n");
-  printf("\n");
-  printf("You should have received a copy of the GNU General Public License"\
-	 "\n");
-  printf("along with this program; if not, write to the Free Software\n");
-  printf("Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");
+#define COPYRIGHT "\
+This program is free software; you can redistribute it and/or modify\n\
+it under the terms of the GNU General Public License as published by\n\
+the Free Software Foundation; either version 2 of the License, or\n\
+(at your option) any later version.\n\
+\n\
+This program is distributed in the hope that it will be useful,\n\
+but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
+GNU General Public License for more details.\n\
+\n\
+You should have received a copy of the GNU General Public License\n\
+along with this program; if not, write to the Free Software Foundation,\n\
+Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n"
 
+  printf(COPYRIGHT);
   return 0.0; /* dummy value */
 }
 
@@ -111,38 +118,37 @@ static int cmp(const void *f1, const void *f2)
 }
 
 /* Evaluate the command line. The work is done in this procedure ! */
-int eval(char line[])
+static int eval(char line[])
 {
-  short int quit;		/* flags */
-  static char delimiters[] = " ,;\t\n\0";
+  int quit;		/* flags */
+  char delimiters[] = " ,;\t\f\n";
   char *cmnd, *rest;		 /* one command line */
   double op1, op2, res;		 /* operands */
 
-  cmnd = strtok(line, delimiters); res = 0; pushtostack=TRUE;
-  do
-  {
-    quit = ((strncmp(cmnd, "quit", 4) == 0) || (strncmp(cmnd,"q",1) == 0) || 
-	    (strncmp(cmnd, "exit", 4) == 0));
+  cmnd = strtok(line, delimiters); res = 0; pushtostack=1;
+  quit = ((cmnd == NULL) ||
+  	  (strncmp(cmnd, "quit", 4) == 0) || (strncmp(cmnd,"q",1) == 0) || 
+	  (strncmp(cmnd, "exit", 4) == 0));
 
-    if ((!quit) && (cmnd != NULL))
-    {
+  while (!quit)
+  {
       /* Is it a number ? It yes, push it onto the stack. */      
 
       /* First assume it's a double */
       op1 = strtod(cmnd, &rest);
       if (strcmp(rest, "") == 0) 
       {
-	push(op1); /* It is a float */
+	(void)push(op1); /* It is a float */
       }
       else
       {
 	/* converting number according to the rules of the C programming
 	 * language, e.g 0=octal, 0x=hexadecimal
 	 */
-	op1 = strtol(cmnd, &rest, 0);
+	op1 = (double)strtol(cmnd, &rest, 0);
 	if (strcmp(rest, "") == 0) 
 	{
-	  push(op1); /* it is an integer */
+	  (void)push(op1); /* it is an integer */
 	}
 	else
 	{
@@ -169,60 +175,96 @@ int eval(char line[])
 	    if (errno != 0) { perror(NULL); }
 	    else
 	    {
-	      if (f->pushrestostack && pushtostack) push(res);
+	      if (f->pushrestostack && pushtostack) (void)push(res);
 	    }
 	  }
 	}
       }
       cmnd = strtok(NULL, delimiters);
-    }
+      quit = ((cmnd == NULL) ||
+  	      (strncmp(cmnd, "quit", 4) == 0) || (strncmp(cmnd,"q",1) == 0) || 
+  	      (strncmp(cmnd, "exit", 4) == 0));
+
   }
-  while ((!quit) && (cmnd != NULL));
     
-  return quit;
+  /* return quit; */
+  return (cmnd != NULL);
 }
 
-unsigned char *getline(int interactive)
-/* Read a line of text without imposing an length limit */
-{
-  unsigned char *line=NULL;
-  int c;
-  int i;
-
-#ifdef HAVE_READLINE
-  if (interactive) line=(unsigned char*)readline(""); 
-  else
-  {
-#endif
-    i=0; line=xmalloc(LINEFRAGMENT+1);
-    while (((c = getc(stdin)) != EOF) && (c != '\n') && (c != '\0'))
-    {
-     if ((i > LINEFRAGMENT) &&  (i % LINEFRAGMENT) == 0) 
-       line=xrealloc(line,i+LINEFRAGMENT+1);
-     line[i]=c; i++;
-    }
-    if ((c != EOF) && (c != '\0')) line[i]='\0';
-    else
-    {
-      free(line); line=NULL; /* this is readline compatible; 
-				return NULL when the input line is empty */
-    }
-#ifdef HAVE_READLINE
-  }
-#endif
-  return line;
-}
 /* print a short usage statement. Indicate that the argument must be quoted. */
-void usage(void)
+static void usage(void)
 {
   fprintf(stderr, "usage: %s [-h][-v] [\"expression\"]\n", progname);
 }
 
+#ifdef HAVE_READLINE
+/* the initialisation code was taken from:
+     fileman.c -- A tiny application which demonstrates how to use the
+     GNU Readline library.  This application interactively allows users
+     to manipulate files and their modes. 
+ */
+
+/* Attempt to complete on the contents of TEXT.  START and END show the
+   region of TEXT that contains the word to complete.  We can use the
+   entire line in case we want to do some simple parsing.  Return the
+   array of matches, or NULL if there aren't any. */
+
+char **rpncalc_completion(char *text, int start, int end)
+{
+  return completion_matches (text, command_generator);
+}
+
+/* Generator function for command completion.  STATE lets us know whether
+   to start from scratch; without any state (i.e. STATE == 0), then we
+   start at the top of the list. */
+char *command_generator (char *text, int state)
+{
+  static int list_index, len;
+  char *name;
+
+  /* If this is a new word to complete, initialize now.  This includes
+     saving the length of TEXT for efficiency, and initializing the index
+     variable to 0. */
+  if (!state) { list_index=0; len=strlen (text); }
+
+  /* Return the next name which partially matches from the command list. */
+  while (list_index < NCMDTAB)
+  {
+    name = cmdtab[list_index].fname; list_index++;
+
+    if (strncmp (name, text, len) == 0) return (dupstr(name));
+  }
+
+  /* If no names matched, then return NULL. */
+  return ((char *)NULL);
+}
+
+/* Tell the GNU Readline library how to complete.  We want to try to complete
+   on command names if this is the first word in the line. */
+void initialize_readline(int interactive)
+{
+  /* Allow conditional parsing of the ~/.inputrc file. */
+  rl_readline_name = "rpncalc";
+
+  /* Tell the completer that we want a crack first. */
+  if (interactive)
+  {
+    rl_attempted_completion_function = (CPPFunction *)rpncalc_completion;
+  }
+  else
+  {
+    rl_bind_key('\t', rl_insert()); 
+    /* Suppress key binding of TAB in batch mode */
+  }
+}
+
+#endif
+
 int main(int argc, char *argv[])
 {
   int c, i;
-  char *line=NULL;		/* entire line  */
-  short int quit;		/* should we quit ? */
+  char *line=NULL;	/* entire line  */
+  int quit;			/* should we quit ? */
 
   progname=strrchr(argv[0],'/');
   if (progname==NULL) progname=argv[0];
@@ -235,9 +277,8 @@ int main(int argc, char *argv[])
     {
       case 0  : break;
       case 'h':
-      case '?': usage(); return 0; break;
-      case 'v': fprintf(stderr,"%s version 1.0\n", progname); 
-	        return 1; break;
+      case '?': usage(); return 0;
+      case 'v': fprintf(stderr,"%s version 1.0\n", progname); return 1;
       default : break;
     }
   }
@@ -256,15 +297,14 @@ int main(int argc, char *argv[])
     short int l;
 
     l=0; for(i=optind;i<argc;i++) l += (strlen(argv[optind])+1);
-    line=(unsigned char *)xmalloc(l+1);
+    line=(char *)xmalloc(l+1);
     strcpy(line, argv[optind]);
     for (i = optind+1; i < argc; i++)
     {
       strcat(line, " "); strcat(line, argv[i]);
     }
 
-    eval(line);    
-    showstack();
+    quit=eval(line); (void)showstack();
     free(line); line=NULL;
   }
   else
@@ -272,6 +312,10 @@ int main(int argc, char *argv[])
     int interactive;
 
     interactive=isatty(0); line=NULL;
+
+#ifdef HAVE_READLINE    
+    initialize_readline(interactive);
+#endif
     if (interactive)
     {
       fprintf(stderr,"For details, type `warranty'.\n");
@@ -284,7 +328,7 @@ int main(int argc, char *argv[])
 
     do
     {
-      line=getline(interactive);
+      line=getline();
       quit=(line == NULL);
       if (!quit)
       {
@@ -293,17 +337,17 @@ int main(int argc, char *argv[])
 	if (*line != '\0')
 	{
 #ifdef HAVE_READLINE          
-	  if (interactive) add_history(line);
+	  add_history(line);
 #endif
-	  quit = eval(line);
-	  if (!quit) showstack();
+	  quit=eval(line);
+	  if (!quit) (void)showstack();
 	}
       }
       free(line); line=NULL; /* line was malloc'd by readline/getline. */
     }
     while (!quit);
 #ifdef HAVE_READLINE 
-    if (interactive) remove_history();
+    remove_history();
 #endif    
   }  
   return 0;
